/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
draw.cpp

  Misc drawing routines.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _DRAW_CPP_
#include "draw.h"

void transformSquarify(ptTransform2& tr, double x, double y)
{
	if ((x < 1.0) || (y < 1.0))
	{
		tr.scale = Point2(1.0, 1.0);
		tr.origin = Point2(0.0);
		return;
	}

	if (x > y)
	{
		// 2 => y
		tr.scale = Point2(y, y) / 2;

		// (-1,-1) => ((x - y) / 2, 0)
		tr.origin = Point2(-1.0 - (x - y) / y, -1.0);
	} else {
		// 2 => x
		tr.scale = Point2(x, x) / 2;

		// (-1,-1) => (0, (y - x) / 2)
		tr.origin = Point2(-1.0, -1.0 - (y - x) / x);
	}
}
void transformMakeFit(ptTransform2& tr, double x, double y, double sx, double sy)
{
	if ((x < 1.0) || (y < 1.0))
	{
		tr.scale = Point2(1.0, 1.0);
		tr.origin = Point2(0.0);
		return;
	}

	if ((x / sx) > (y / sy))
	{
		// sy => y
		tr.scale = Point2(y, y) / sy;

		// (0,0) => ((x - sx*scale) / 2, 0)
		tr.origin = Point2(-(x/tr.scale.x - sx) / 2.0, 0.0);
	} else {
		// sx => x
		tr.scale = Point2(x, x) / sx;

		// (0,0) => (0, (y - sy*scale) / 2)
		tr.origin = Point2(0.0, -(y/tr.scale.y - sy) / 2.0);
	}
}

// r,theta => x,y
Point2 unPolar(Point2 rad)
{
	return Point2(rad.x * sin(rad.y), rad.x * -cos(rad.y));
}

/*

  Curve drawing

 */
void drawCircle(HDC hDC, ptTransform2& tr, Point2 a, Point2 b)
{
	Point2 A,B;

	A = transform(tr, a);
	B = transform(tr, b);
	Ellipse(hDC, int(A.x), int(A.y), int(B.x), int(B.y));
}
// a,b = bounding box, from, to = radials
void drawArc(HDC hDC, ptTransform2& tr, Point2 a, Point2 b, Point2 from, Point2 to)
{
	Point2 A,B,F,T;

	A = transform(tr, a);
	B = transform(tr, b);
	F = transform(tr, from);
	T = transform(tr, to);
	Arc(hDC, int(A.x), int(A.y), int(B.x), int(B.y), int(F.x), int(F.y), int(T.x), int(T.y));
}
// note: angles in degrees
void drawWedge(HDC hDC, ptTransform2& tr, double thStart, double thEnd, 
			   double r1, double r2, double resolution)
{
	double th1, th2;
	double th1d, th2d;

	for (th1d = thStart; th1d < thEnd; th1d += resolution)
	{
		th2d = th1d + resolution;
		if (th2d > thEnd) th2d = thEnd;
		th1 = th1d * PI / 180.0;
		th2 = th2d * PI / 180.0;

		drawQuad(hDC, tr, unPolar(Point2(r1, th1)), unPolar(Point2(r2, th1)),
			unPolar(Point2(r2, th2)), unPolar(Point2(r1, th2)));
	}
}
void drawBezier(HDC hDC, ptTransform2& tr, Point2 start, Point2 c1, Point2 c2, Point2 end)
{
	Point2 A,B,C,D;
	POINT pts[4];

	A = transform(tr, start);
	B = transform(tr, c1);
	C = transform(tr, c2);
	D = transform(tr, end);

	pts[0].x = int(A.x);
	pts[0].y = int(A.y);
	pts[1].x = int(B.x);
	pts[1].y = int(B.y);
	pts[2].x = int(C.x);
	pts[2].y = int(C.y);
	pts[3].x = int(D.x);
	pts[3].y = int(D.y);

	PolyBezier(hDC, pts, 4);
}

/*

  Line/polygon drawing

 */
void drawLine(HDC hDC, ptTransform2& tr, Point2 a, Point2 b)
{
	Point2 A,B;

	A = transform(tr, a);
	B = transform(tr, b);
	MoveToEx(hDC, int(A.x), int(A.y), 0);
	LineTo(hDC, int(B.x), int(B.y));
}
void drawRect(HDC hDC, ptTransform2& tr, Point2 a, Point2 b)
{
	Point2 A,B;

	A = transform(tr, a);
	B = transform(tr, b);
	Rectangle(hDC, int(A.x), int(A.y), int(B.x), int(B.y));
}
void drawQuad(HDC hDC, ptTransform2& tr, Point2 a, Point2 b, Point2 c, Point2 d)
{
	Point2 A,B,C,D;
	POINT pts[4];

	A = transform(tr, a);
	B = transform(tr, b);
	C = transform(tr, c);
	D = transform(tr, d);

	pts[0].x = int(A.x);
	pts[0].y = int(A.y);
	pts[1].x = int(B.x);
	pts[1].y = int(B.y);
	pts[2].x = int(C.x);
	pts[2].y = int(C.y);
	pts[3].x = int(D.x);
	pts[3].y = int(D.y);

	Polygon(hDC, pts, 4);
}

/*

  Pixel/dot drawing

 */
void drawSetPixel(HDC hDC, ptTransform2& tr, Point2 pt, COLORREF clr)
{
	Point2 A;

	A = transform(tr, pt);
	SetPixel(hDC, int(A.x), int(A.y), clr);
}
void drawDot(HDC hDC, ptTransform2& tr, Point2 pt)
{
	Point2 A;

	A = transform(tr, pt);
	Rectangle(hDC, int(A.x) - 1, int(A.y) - 1, int(A.x) + 1, int(A.y) + 1);
}
void drawFloodFill(HDC hDC, ptTransform2& tr, Point2 pt, COLORREF clr, int clrIsBorder)
{
	Point2 A;

	A = transform(tr, pt);
	ExtFloodFill(hDC, int(A.x), int(A.y), clr, 
		clrIsBorder ? FLOODFILLBORDER : FLOODFILLSURFACE);
}

/*

  Text drawing

 */
HFONT drawMakeFont(HDC hDC, ptTransform2& tr, double Sz)
{
	LOGFONT lf;
	HFONT hFont;

	memset(&lf, 0, sizeof(lf));
	
	lf.lfCharSet		= ANSI_CHARSET;
	lf.lfOutPrecision	= OUT_TT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= DEFAULT_PITCH | FF_DONTCARE;

	lf.lfHeight			= - int(Sz * tr.scale.y);
	strcpy(lf.lfFaceName, "Arial");

	hFont = CreateFontIndirect(&lf);
	SelectObject(hDC, hFont);

	return hFont;
}
void drawText(HDC hDC, ptTransform2& tr, Point2 pt, char *text)
{
	Point2 p;
	RECT r, r2;

	memset(&r, 0, sizeof(r));
	DrawText(hDC, text, strlen(text), &r, DT_CALCRECT | DT_NOPREFIX | DT_SINGLELINE);

	p = transform(tr, pt);
	r2.left		= int(p.x) - r.right / 2;
	r2.right	= int(p.x) + r.right / 2;
	r2.top		= int(p.y) - r.bottom / 2;
	r2.bottom	= int(p.y) + r.bottom / 2;
	DrawText(hDC, text, strlen(text), &r2, 
		DT_CENTER | DT_VCENTER | DT_NOPREFIX | DT_SINGLELINE | DT_NOCLIP);
}
