/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
packet.cpp

  Telemetry packet processing.
*/

#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define _PACKET_CPP_
#include "packet.h"
#include "uavgs.h"
#include "sepstr.h"

volatile int nTelemThreads = 0, terminate = 0;

int nVars = 0;
CVariable *vars[256];

DWORD WINAPI threadComPort(LPVOID lpvTelem)
{
	CTelemetry *telem;
	char packet[256];
	int packetPos = 0;
	char chr;
	DWORD nRead;

	nTelemThreads++;

	telem = (CTelemetry*) lpvTelem;

	while (!terminate)
	{
		if(!ReadFile(telem->hCom, &chr, 1, &nRead, 0))
			continue;
		if (!nRead)
			continue;
		
		if (chr == telem->chrStart)
		{
			// start
			packetPos = 0;
			memset(packet, 0, sizeof(packet));
		} else if (chr == telem->chrEnd) {
			// end
			packet[packetPos] = chr;
			packet[packetPos+1] = 0;
			packetPos = 0;
			
			telem->processPacket(packet);
		} else {
			packet[packetPos] = chr;
			if (packetPos < 256) packetPos++;
		}
	}

	nTelemThreads--;

	return 0;
}

CVariable *getVar(char *name)
{
	int i;
	CVariable *v;

	for (i = 0; i < nVars; i++)
	{
		if (!stricmp(vars[i]->name, name))
			return vars[i];
	}

	// not found - create one
	v = new CVariable;
	vars[nVars] = v;
	nVars++;
	strcpy(v->name, name);
	return v;
}
void deleteVars()
{
	int i;

	for (i = 0; i < nVars; i++)
		delete vars[i];

	nVars = 0;
}

/*

  CVariable

 */
CVariable::CVariable()
{
	name[0] = 0;
	type = VART_NUMBER;
	nWindow = 0;

	// value MUST be initialized here, before type is even known,
	// or stuff may get erased when the script loads
	valueNum = 0.0;
	valueStr[0] = 0;
}
CVariable::~CVariable()
{
}
void CVariable::set(double value)
{
	int i;

	switch (type)
	{
	case VART_NUMBER:
	case VART_GPSLAT:
	case VART_GPSLON:
		valueNum = value;
		break;
	case VART_STRING:
		sprintf(valueStr, "%g", value);
		break;
	}

	// update windows
	for (i = 0; i < nWindow; i++)
	{
		wnd[i]->update(this);
	}
}
void CVariable::set(char *value)
{
	int i;
	char a[256], b[256], c[256];
	double d, m;

	switch (type)
	{
	case VART_NUMBER:
		valueNum = atof(value);
		break;
	case VART_STRING:
		strcpy(valueStr, value);
		break;
	case VART_GPSLAT:
	case VART_GPSLON:
		// [D]DDMM "." M[M[...]] "," {N | S | E |W}
		SepStr(value, ',', a, c);
		
		// find the decimal point
		for (i = 0; a[i] && (a[i] != '.'); i++);
		i = max(i - 2, 0);
		strcpy(b, a + i);
		a[i] = 0;

		d = atof(a);
		m = atof(b);
				
		if (!stricmp(c, "S") || !stricmp(c, "W"))
			valueNum = -1.0;
		else if (!stricmp(c, "N") || !stricmp(c, "E"))
			valueNum = 1.0;
		else
			valueNum = 0.0;

		valueNum *= d + m / 60.0;
		break;
	}

	// update windows
	for (i = 0; i < nWindow; i++)
	{
		wnd[i]->update(this);
	}
}
double CVariable::getNumber()
{
	switch (type)
	{
	case VART_NUMBER:
	case VART_GPSLAT:
	case VART_GPSLON:
		return valueNum;
	}
	return 0.0;
}
char * CVariable::getString()
{
	switch (type)
	{
	case VART_NUMBER:
		sprintf(valueStr, "%g", valueNum);
		return valueStr;
	case VART_STRING:
		return valueStr;
	case VART_GPSLAT:
		degMakeDMS(valueNum, valueStr + 2);
		valueStr[0] = (valueNum >= 0.0) ? 'N' : 'S';
		valueStr[1] = ' ';
		return valueStr;
	case VART_GPSLON:
		degMakeDMS(valueNum, valueStr + 2);
		valueStr[0] = (valueNum >= 0.0) ? 'E' : 'W';
		valueStr[1] = ' ';
		return valueStr;
	}
	return "?";
}

void CVariable::addWindow(CWndFrameObj *w)
{
	wnd[nWindow] = w;
	nWindow++;
}
/*

  CTelemetry

 */
void CTelemetry::processPacket(char *packet)
{
	int i;
	CPacketType *type;
	char *pos, *strpos, str[256];
	union {
		__int8 s8;
		unsigned __int8 u8;
	};
	union {
		__int16 s16;
		unsigned __int16 u16;
	};

	type = 0;
	for (i = 0; (i < nPType) && !type; i++)
		if (ptype[i]->isMatch(packet))
			type = ptype[i];
	if (!type) 
		return; // invalid (or just unrecognized) packet

	pos = packet + strlen(type->start);
	
	// decode the packet's items
	for (i = 0; i < type->nItem; i++)
	{
		switch (type->item[i]->type)
		{
		case PKTI_MESSAGE:
			if (chrEnd)
			{
				// strip off packet end character
				packet[strlen(packet) - 1] = 0;
			}
			message(pos);

			// (can be nothing after this)
			return;
		case PKTI_STRING:
			strpos = str;
			while (*pos && (*pos != ' ') && (*pos != ',') && (*pos != 10) && (*pos != 13)
				   && (*pos != chrEnd))
			{
				*strpos++ = *pos++;
			}
			if (*pos)
				pos++;
			*strpos = 0;

			if (type->item[i]->var) 
				type->item[i]->var->set(str);
			break;
		case PKTI_NUMBER:
			strpos = str;
			while (*pos && (*pos != ' ') && (*pos != ',') && (*pos != 10) && (*pos != 13)
				   && (*pos != chrEnd))
			{
				*strpos++ = *pos++;
			}
			if (*pos)
				pos++;
			*strpos = 0;

			if (type->item[i]->var)
				type->item[i]->var->set(atof(str) * type->item[i]->scale);
			break;
		case PKTI_BINARY:
			// now things are going to get a teeeensy bit complicated!
			if (type->item[i]->details & PKTI_BINARY_32)
			{
				message("!Warning 32 bit numbers not working yet!");

				pos += 4;
			} else if (type->item[i]->details & PKTI_BINARY_16) {
				if (type->item[i]->details & PKTI_BINARY_MSB1)
				{
					u16 =
						( ((unsigned int) * ((unsigned char *) pos)) << 8 ) +
						  (* ((unsigned char *) (pos + 1)));
				} else {
					u16 =
						  (* ((unsigned char *) pos)) +
						( ((unsigned int) * ((unsigned char *) (pos + 1))) << 8 );
				}

				if (type->item[i]->details & PKTI_BINARY_SIGN)
				{
					if (type->item[i]->var) 
						type->item[i]->var->set(double(s16) * type->item[i]->scale);
				} else {
					if (type->item[i]->var)
						type->item[i]->var->set(double(u16) * type->item[i]->scale);
				}
				
				pos += 2;
			} else {
				s8 = *pos;
				if (type->item[i]->details & PKTI_BINARY_SIGN)
				{
					if (type->item[i]->var) 
						type->item[i]->var->set(double(s8) * type->item[i]->scale);
				} else {
					if (type->item[i]->var)
						type->item[i]->var->set(double(u8) * type->item[i]->scale);
				}

				pos ++;
			}
			break;
		case PKTI_GPSLAT:
		case PKTI_GPSLON:
			// Two parts to it - number then N/S or E/W
			strpos = str;
			while (*pos && (*pos != ' ') && (*pos != ',') && (*pos != 10) && (*pos != 13)
				   && (*pos != chrEnd))
			{
				*strpos++ = *pos++;
			}
			if (*pos)
				pos++;

			*strpos++ = ',';

			while (*pos && (*pos != ' ') && (*pos != ',') && (*pos != 10) && (*pos != 13)
				   && (*pos != chrEnd))
			{
				*strpos++ = *pos++;
			}
			*strpos = 0;

			if (*pos)
				pos++;

			if (type->item[i]->var) 
				type->item[i]->var->set(str);
			break;
		}
	}
}

int CTelemetry::initComPort(char *comPort)
{
	DCB dcb;
	DWORD dw;
	char str[256];
	COMMTIMEOUTS ct;

	// Open COM port
	hCom = CreateFile(comPort, GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0);
	if (hCom == INVALID_HANDLE_VALUE)
	{
		sprintf(str,
			"Cannot open COM port %s. Make sure it is not being used "
			"by another program.", comPort);
		fatalError(str);
		return 0;
	}
		
	memset(&dcb, 0, sizeof(dcb));
	dcb.DCBlength	= sizeof(dcb);
	dcb.fBinary		= 1;	
	// set baud rate
	dcb.BaudRate	= comBaud;
	// 8N1 - 8 data bits, no parity, 1 stop bit
	dcb.ByteSize	= 8;
	dcb.Parity		= NOPARITY;
	dcb.StopBits	= ONESTOPBIT;

	if (!SetCommState(hCom, &dcb))
	{
		fatalError("Error initializing COM port: SetCommState() failed.");
		return 0;
	}

	if (!SetCommMask(hCom, EV_RXCHAR | EV_RXFLAG))
	{
		fatalError("Error initializing COM port: SetCommMask() failed.");
		return 0;
	}

	// timeout setup errors are annoying but not fatal
	if (!GetCommTimeouts(hCom, &ct))
	{
		error("Error initializing COM port: GetCommTimeouts() failed.");
	} else {
		ct.ReadIntervalTimeout			= 500;
		ct.ReadTotalTimeoutMultiplier	= 500;
		ct.ReadTotalTimeoutConstant		= 500;
		if (!SetCommTimeouts(hCom, &ct))
		{
			error("Error initializing COM port: SetCommTimeouts() failed.");
		}
	}

	CreateThread(0, 0, threadComPort, (LPVOID) this, 0, &dw);

	return 1;
}
void CTelemetry::stopComPort()
{
	CloseHandle(hCom);
}

int CTelemetry::initTelemetry()
{
	if (!initComPort(comPort))
	{
		message("!Warning: initComPort() failed");
		// return 0;
	}
	
	return 1;
}
void CTelemetry::stopTelemetry()
{
	terminate = 1;
	stopComPort();
}

CTelemetry::CTelemetry()
{
	chrStart = chrEnd = 0;
	comBaud = 0;
	strcpy(comPort, "<port unspecified>");
	nPType = 0;
}
CTelemetry::~CTelemetry()
{
	int i;

	stopTelemetry();

	for (i = 0; i < nPType; i++)
		delete ptype[i];
}

/*

  CPacketType

 */
CPacketType::CPacketType()
{
	start[0] = 0;
	nItem = 0;
}
CPacketType::~CPacketType()
{
	int i;

	for (i = 0; i < nItem; i++)
		delete item[i];
}
int CPacketType::isMatch(char *packet)
{
	int i;

	for (i = 0; start[i]; i++)
	{
		if (!packet[i])
			return 0;	// too short

		if (packet[i] != start[i]) 
			return 0;	// doesn't match
	}

	return 1; // hooray!
}

/*

  CPacketItem

 */
CPacketItem::CPacketItem()
{
	type = details = 0;
	var = 0;
	scale = 1.0;
}
CPacketItem::CPacketItem(int _type, int _details, char *varName)
{
	type = _type;
	details = _details;
	scale = 1.0;
	
	if (varName)
	{
		var = getVar(varName);

		switch (type)
		{
		case PKTI_BINARY: case PKTI_NUMBER:
			var->type = VART_NUMBER;
			var->set(var->getNumber());
			break;
		case PKTI_STRING:
			var->type = VART_STRING;
			var->set(var->getString());
			break;
		case PKTI_GPSLAT:
			var->type = VART_GPSLAT;
			var->set(var->getNumber());
			break;
		case PKTI_GPSLON:
			var->type = VART_GPSLON;
			var->set(var->getNumber());
			break;
		}
	} else
		var = 0; // not associated with a variable
}
CPacketItem::~CPacketItem()
{
}

