/*

UAV Ground Station model aircraft wing design tool
Copyright (C) 2003 Robin Stanton

This file is part of UAV Ground Station.

UAV Ground Station is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UAV Ground Station is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UAV Ground Station; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/*
 Point.cpp

  Classes for working with points and vectors. Supports
  addition, subtraction, multiplication, and division.
 */

#include <math.h>	// Square roots, etc.
#include "point.h"

/*
 Point2 Functions
*/
Point2::Point2()
{
	x = 0.0;
	y = 0.0;
}
Point2::Point2( const double _x, const double _y )
{
	x = _x;
	y = _y;
}
Point2::Point2( const double values[2] )
{
	x = values[0];
	y = values[1];
}
Point2::Point2( const double value )
{
	x = value;
	y = value;
}
Point2::~Point2()
{
}

// Operators
Point2 Point2::operator + ( const Point2 P ) const
{
	return Point2( x + P.x, y + P.y );
}
Point2 Point2::operator - ( const Point2 P ) const
{
	return Point2( x - P.x, y - P.y );
}
Point2 Point2::operator * ( const Point2 P ) const
{
	return Point2( x * P.x, y * P.y );
}
Point2 Point2::operator / ( const Point2 P ) const
{
	return Point2( x / P.x, y / P.y );
}

// Functions
double Point2::Length () const
{
	return sqrt( x*x + y*y );
}
double Point2::LengthSquared () const
{
	return x*x + y*y;
}


/*
 Point3 Functions
*/
Point3::Point3()
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
}
Point3::Point3( const double _x, const double _y, const double _z )
{
	x = _x;
	y = _y;
	z = _z;
}
Point3::Point3( const double values[3] )
{
	x = values[0];
	y = values[1];
	z = values[2];
}
Point3::Point3( const double value )
{
	x = value;
	y = value;
	z = value;
}
Point3::Point3( Point2 P )
{
	x = P.x;
	y = P.y;
	z = 0.0;
}
Point3::~Point3()
{
}

// Operators
Point3 Point3::operator + ( const Point3 P ) const
{
	return Point3( x + P.x, y + P.y, z + P.z );
}
Point3 Point3::operator - ( const Point3 P ) const
{
	return Point3( x - P.x, y - P.y, z - P.z );
}
Point3 Point3::operator * ( const Point3 P ) const
{
	return Point3( x * P.x, y * P.y, z * P.z );
}
Point3 Point3::operator / ( const Point3 P ) const
{
	return Point3( x / P.x, y / P.y, z / P.z );
}

// Functions
double Point3::Length () const
{
	return sqrt( x*x + y*y + z*z );
}
double Point3::LengthSquared () const
{
	return x*x + y*y + z*z;
}


double DotProduct( const Vector3 A, const Vector3 B )
{
	return (A.x * B.x) + (A.y * B.y) + (A.z * B.z);
}

Vector3 CrossProduct( const Vector3 A, const Vector3 B )
{
    return Point3( A.y*B.z - A.z*B.y,
                   A.z*B.x - A.x*B.z,
                   A.x*B.y - A.y*B.x );
}

void Normalize( Vector3 *V )
{
	*V = *V / Vector3( (double) V->Length() );
}
Vector3 Normalize( Vector3 V )
{
	return V / Vector3( (double) V.Length() );
}

Point2 transform(ptTransform2& tr, Point2 p)
{
	return (tr.scale * (p - tr.origin));
}
Point2 unTransform(ptTransform2& tr, Point2 p)
{
	return ((p / tr.scale) + tr.origin);
}

/*
 Point2I Functions
*/
Point2I::Point2I()
{
	x = 0;
	y = 0;
}
Point2I::Point2I( const int _x, const int _y )
{
	x = _x;
	y = _y;
}
Point2I::Point2I( const int values[2] )
{
	x = values[0];
	y = values[1];
}
Point2I::Point2I( const int value )
{
	x = value;
	y = value;
}
Point2I::~Point2I()
{
}

// Operators
Point2I Point2I::operator + ( const Point2I P ) const
{
	return Point2I( x + P.x, y + P.y );
}
Point2I Point2I::operator - ( const Point2I P ) const
{
	return Point2I( x - P.x, y - P.y );
}
Point2I Point2I::operator * ( const Point2I P ) const
{
	return Point2I( x * P.x, y * P.y );
}
Point2I Point2I::operator / ( const Point2I P ) const
{
	return Point2I( x / P.x, y / P.y );
}

// Functions
double Point2I::Length () const
{
	return sqrt( double(x)*double(x) + double(y)*double(y) );
}
double Point2I::LengthSquared () const
{
	return double(x)*double(x) + double(y)*double(y);
}

